##############################################################
# dilutionFit.R
#
# This is the result of the model building in doSimulation.R
# Original version by Kevin R. Coombes and Jianhua Hu.
# Modified by Corwin Joy.
# Modified by: Kevin R. Coombes, 31 March 2006
# Modified by: Corwin Joy, 31 March 2006 to add more robust estimate for
#              alpha, beta & more formal regression tests.
# Last modified by: Kevin Coombes, 3 May 2006
#   Converted to S4 classes. Completely revised interface, giving more
#   sensible names to inputs and outputs. Added numerous useful methods
#   for assessing the quality of the fit.

setClass("RPPAFit",
         representation=list(
           call = "call",             # function call used to create the model
           rppa = "RPPA",               # required parameter
           design ="RPPADesign",        # required parameter
           measure = "character",       # required parameter
           method = "character",        # optional parameter
           coefficients = "numeric",  # alpha, beta, gamma
           concentrations = "numeric",# main output
           lower = "numeric",           # confidence interval 
           upper = "numeric",           # confidence interval
           conf.width = "numeric",      # width of confidence interval
           intensities = "numeric",   # intensities related to series concentrations
           linear.p = "numeric",      # is the series increasing?
           slope = "numeric",           # ditto
           p.values = "numeric",      # measure of goodness of fit per dilution series
           ss.ratio = "numeric",
           warn = "character",
           version = "character"
           ))

setMethod("summary", "RPPAFit", function(object, ...) {
  cat(paste("An RPPAFit object constructed via the function call:\n",
            as.character(list(object@call)), "\n"))
})

# The coefficients are 'alpha', 'beta', and 'gamma'
setMethod("coef", "RPPAFit", function(object, ...) {
  object@coefficients
})

setMethod("coefficients", "RPPAFit", function(object, ...) {
  object@coefficients
})

# We are actually interested in estimating the concentrations for each
# dilution series (which may be the same as a sample). However, when we do
# that we also get estimated concentrations at each spot, with corresponding
# predicted intensities. Default for 'fitted' is to return the per-spot
# fitted 'Y' intensities, with an option to return the per-spot fitted
# 'X' concentrations.
setMethod("fitted", "RPPAFit", function(object,
                                            type=c("Y", "y", "X", "x"),
                                            ...) {
  type <- match.arg(type)
  conc <- object@concentrations
  series <- as.character(object@design@layout$Series)
  fitX <- conc[series] + object@design@layout$Steps
  if(type=="x" | type == "X") {
    return(fitX)
  }
  cf <- as.list(object@coefficients)
  cf$alpha + cf$beta*.ea(cf$gamma*fitX)
})

# raw residuals are defined so that
#    observed intensity = fitted Y + raw residuals
# standardized residuals are formed from the raw residuals in the obvious way
# linear residuals are the residuals after the logistic transformation
#
# Note that the model fitting is a bit of a hybrid between the linear and
# logistic intensity scales, so it's not completely clear which residuals are
# most meaningful
setMethod("residuals", "RPPAFit", function(object,
                                               type=c("raw", "standardized", "linear"),
                                               ...) {
  type <- match.arg(type)
  res <- object@rppa@data[, object@measure] - fitted(object)
  if (type == "standardized") {
    res <- scale(res)
  }
  if (type == "linear") {
    cf   <- as.list(coef(object))
    y    <- object@rppa@data[, object@measure]
    xhat <- fitted(object, "X")
    warn <- options(warn=-1)
    res  <- .lp((y-cf$alpha)/cf$beta) - cf$gamma*xhat
    options(warn)
  }
  res
})

setMethod("resid", "RPPAFit", function(object, ...) {
  residuals(object, ...)
})

# The image function gives a geographic plot of some measure computed from
# the fit. Default is to image the (raw) residuals, with options for other
# forms of the residuals or for the fitted concentrations (X) or intensities (Y).
setMethod("image", "RPPAFit", function(x,
                                           measure=c("Residuals", "StdRes", "LinRes", "X", "Y"),
                                           ...) {
  measure <- match.arg(measure)
  rppa <- x@rppa
  rppa@data$Residuals <- resid(x)
  rppa@data$StdRes <- resid(x, "standardized")
  rppa@data$LinRes <- resid(x, "linear")
  rppa@data$X = fitted(x, "X")
  rppa@data$Y = fitted(x, "Y")
  image(rppa, measure=measure, ...)
  invisible(x)
})

# Histogram of the residuals, with an option to see the standardized or linear residuals
setMethod("hist", "RPPAFit", function(x,
                                          type=c("Residuals", "StdRes", "LinRes"),
                                          xlab=NULL, main=NULL, ...) {
  type <- match.arg(type)
  if(is.null(xlab)) xlab <- type
  if (is.null(main)) main <- paste('Histogram of', type)
  translate <- c("raw", "standardized", "linear")
  names(translate) <- c("Residuals", "StdRes", "LinRes")
  res <- resid(x, type=translate[type])
  hist(res, xlab=xlab, main=main, ...)
})

# Plot the results from dilutionFit above to see how well supercurve fits
setMethod("plot", "RPPAFit", function(x, y,
                                          type=c("cloud", "series", "individual"),
                                          xlab='Log Concentration',
                                          ylab='Intensity', colors=NULL, ...) {
  type <- match.arg(type)
  xval <- fitted(x, 'x')
  yval <- fitted(x, 'y')
  yraw <- x@rppa@data[, x@measure]
  series <- seriesNames(x@design)
  steps <- getSteps(x@design)
  if (type == "cloud" | type == "series") {
    plot(xval, yraw, xlab=xlab, ylab=ylab, ...)
    lines(sort(xval),sort(yval))
    if (type == "series") {
      if (is.null(colors)) {
        if(!require(colorspace)) {
          colors <- gray(0:7 / 7)
        } else {
          colors <- hcl(seq(0, 360, length=9)[1:8], l=65, c=65)
        }
      }
      i <- 0
      ncol <- length(colors)
      for (this in series) {
        i <-  (i%%ncol) + 1
        items <- x@design@layout$Series == this
        lines(xval[items], yraw[items], col=colors[i], type='b')
      }
      lines(sort(xval), sort(yval), lwd=2)
    }
  } else { # type == "individual"
    xx <- seq(-10, 10, length=200)
    cf <- as.list(coef(x))
    yy <- cf$alpha + cf$beta*.ea(cf$gamma*xx)
    ymax <- 1.2*cf$beta
    ymin <- cf$alpha - 0.2*cf$beta
    for (this in series) {
      plot(xx, yy, xlab=xlab, ylab=ylab, type='l', main=this, ylim=c(ymin, ymax), ...)
      text(-9, 1.1*cf$beta,
           paste('SS Ratio =', format(x@ss.ratio[this], digits=4)), adj=0)
      points(x@concentrations[this], x@intensities[this], pch=16, col='red')
      items <- x@design@layout$Series == this
      lines(xval[items], yraw[items], type='b')
    }
    lines(sort(xval), sort(yval), lwd=2)
  }
  invisible(x)
})

# REQUIRED INPUTS:
# rppa    = an RPPA object
# design  = a RPPADesign object
# measure = name of column in rppa@data to view as intensity
#
# OPTIONAL INPUTS CONTROLLING THE ALGORITHM:
# method   = how to fit the alpha and beta values. simplest is quantiles, which simply
#     uses the 5th and 95th percentiles. Default is "pure", which just takes wahtever
#     the 'nls' method provides. Last is 'rlm', which performs a logistic transform
#     and tries a robust linear fit.
# ignoreNegative = if true, then alues below zero are converted to NA before estimating
#     alpha and beta. Only matters for the 'quantiles' method.
# ci       = if true, calculate a 90% confidence interval for the LC50 supercurve values
# bayesian = if true, use slide level logistic model as a bayesian prior for fitting
#     individual dilution series rather than an absolute set of constants. In particular
#     the baseline response 'alpha' is treated as a bayesian prior to account for
#     individual series having varying background
#
# OPTIONAL INPUTS CONTROLLING VERBOSITY:
# trace = passed to nls in the 'bayesian' portion
# verbose = logical value; should the function tell you what it is doing
# veryVerbose = logical value; should the function overwhelm you
#               to tell you what it is doing
# warnLevel = used to set the 'warn' option before calling 'rlm'.
#             since this is wrapped in a 'try', it won't cause
#             failure but will give us a chance to figure out
#             which dilution series fail. Setting warnLevel to
#             two or greater may change the computed results.
# balance = optional parameter specifying the spot concentration in each dilution step.
#           By default, a 2x dilution is assumed for each sample. If you want to override
#           this, you will need to provide a matrix(nrow=nsample, ncol=ndilutions)
#           that gives concentrations in each row on a log2 scale centered at 0.  
#
# OUTPUTS:
# An object of the RPPAFit class described above.
#
# N.B.!!!!
# If you change this routine please run dilutionFitRegessionTest() as at
# least a basic test that the routine still runs reasonably correctly
#

# KRC: Added yet another parameter, 'method', to control the ways to fit
# alpha and beta. Since we still don't know which method is best, we should
# make it an option so we (and others) can continue to evaluate it.

RPPAFit <- function(rppa, design, measure, xform=function(x) x, 
                        method=c("pure", "mixed", "quantiles", "rlm"),
                        ci=FALSE, ignoreNegative=TRUE, bayesian=FALSE,
                        trace=FALSE, verbose=FALSE, veryVerbose=FALSE,
                        warnLevel=0) {
  # make sure the MASS library is loaded
  if (!require('MASS'))
    stop("This routine requires the MASS library for the 'rlm' routine.")
  # make sure the input types are sensible
  if(!inherits(rppa, "RPPA"))
    stop("'rppa' must be a valid RPPA object")
  if(!inherits(design, "RPPADesign"))
    stop("'design' must be a valid RPPADesign object")
  if (missing("measure"))
    stop("You must supply the name of the measurement column to fit")
  dn <- dimnames(rppa@data)[[2]]
  temp <- pmatch(measure, dn)
  if (is.na(temp))
    stop("You must supply the name of a valid measurement column to fit")
  if (length(temp) > 1)
    stop("'measure' must identify a unique column of 'rppa'")
  measure <- dn[temp]
   
  method <- match.arg(method)
  call <- match.call()
  intensity <- xform(rppa@data[, measure])

  # determine if any of the dilution series don't appear to be at least
  # roughly increasing. note the magic p-value.
  myScreen <- .dilScreen(intensity, design)
  junk <- myScreen$pvalue > 0.30 | myScreen$slope < 0
  if(verbose) {
    cat(paste('There are', sum(junk), 'bad samples out of',
              length(junk), 'samples.\n'))
  }

  # perform the first pass to initialize the estimates
  first <- .firstPass(intensity, design, ignoreNegative)
  passer    <- first$passer
  gamma.est <- first$gamma
  if (verbose) {
    cat(paste('Completed first pass. Parameters:\n\tlBot =', first$lBot,
              '\n\tlTop =', first$lTop, '\n\tG =', first$gamma, '\n\n'))
  }
  
  # put our current guess at the x and y values into vectors
  # here we must finally explicitly poke into the internals of the RPPADesign
  # class in order to omit the control spots.
  yval <- intensity[!.controlVector(design)]
  xval <- getSteps(design) + passer[names(design)]

  # compute a robust estimate for alpha and beta
  # Tried to use robust 'rnls' from 'sfsmisc' to fit alpha and beta below
  # but found it was still too sensitive to outliers.
  # This is a crude way to get alpha and beta but it seems to be robust.
  lBot <- first$lBot
  lTop <- first$lTop
  names(lBot) <- NULL # needed to avert a warning in the data.frame call below
  names(lTop) <- NULL # same

  # conditional on the initial estimates of the x-values, update the model
  if (method == "quantiles") {
    # We stick with the quantile estimates for lBot, lTop, and only use 'nls'
    # to find the gamma parameter
    dum <- data.frame(xval=xval, yval=yval, alpha=lBot, beta=lTop-lBot)
    nls.model <- try(nls(yval ~ alpha + beta*.ea(gamma*xval), data=dum,
                         start=list(gamma=gamma.est),
                         control=nls.control(maxiter=100), na.action='na.omit'))
    if(is(nls.model, 'try-error')) {
      stop('bailing out of quantiles method') # if this fails, we don't know how to recover
    }
    cf <- coef(nls.model)
    p.alpha <- lBot
    p.beta  <- lTop-lBot
  } else { # nls or rlm
    # in this case, we use 'nls' to find the optimum least squares estimates
    # of all three curve parameters. If you use this option, you probably want
    # to preprocess the data to remove extremely low outliers.
    dum <- data.frame(xval=xval, yval=yval)
    nls.model <- try(nls(yval ~ alpha + beta*.ea(gamma*xval), data=dum,
                         start=list(alpha=lBot, beta=lTop-lBot, gamma=gamma.est),
                         control=nls.control(maxiter=100), na.action='na.omit'))
    if(is(nls.model, 'try-error')) {
      stop('bailing out of nls/rlm method') # if this fails, we don't know how to recover
    }  
    cf <- coef(nls.model)
    p.alpha <- cf['alpha']
    p.beta  <- cf['beta']
  }
  p.gamma <- cf['gamma']
  

  if (verbose) {
    cat("Successfully completed first call to 'nls'.",
        "Updated parameters:\n\talpha =", p.alpha,
        '\n\tbeta  =', p.beta, '\n\tgamma =', p.gamma, '\n\n')
  }
  
  if (method == "rlm") {
    if (verbose) cat("Using 'rlm' to refit values for alpha and beta\n")
    # Try to refit alpha and beta, robustly
    # This still does not give a robust a fit for alpha and beta.
    # Better is to simply use quantiles for these as given by lBot and lTop above.
    expval <- .ea(p.gamma * xval)
    dum2 <- data.frame(expval = expval, yval = yval)
    fit2 <- try(rlm(yval ~ expval, data=dum2))
	  # danger, will robinson!
	  # rlm silently uses na.action = na.omit, and can return a vector that
	  # is shorter than the input vectors. check definition of myresid
	  # below to see how we deal with this issue
    if (is(fit2, 'try-error')) {
      warning('Unable to complete robust fit of alpha and beta')
    } else {
      cf <- coef(fit2)
      p.alpha <- cf[1]
      p.beta  <- cf[2]
      if (verbose) {
        cat("Successfully completed first call to 'rlm'.",
            "Updated parameters:\n\talpha =",
            p.alpha, '\n\tbeta  =', p.beta, '\n\n')
      }
    }
  }

  # Conditional on the current estimates of alpha, beta, and gamma, we
  # perform a separate fit of the EC50 values for each dilution series.
  # If the option 'bayesian' is true, then we allow different alpha's
  # for each series to fine tune the baseline.  If 'bayesian' is false,
  # then we perform a logit transform and compute a robust linear model.
  steps <- getSteps(design)
  series <- seriesNames(design)
  pass2 <- rep(NA, length(series))
  names(pass2) <- series
  pval2  <- pass2
  ss.ratio     <- pass2
  warn2  <- rep('', length(series))
  names(warn2) <- series
  ssy <- -1
  logitz <- .calcLogitz(yval, p.alpha, p.beta, p.gamma)
  for (this in series) {
    items <- names(design) == this
    if (bayesian) {
      # Proper linear bayesian is done by incorporating your prior as an
      # additional set of data points with weights corresponding to
      # confidence in the prior. See
      # http://limnology.wisc.edu/regime/appendix_14jul03.pdf
      # For the moment, I am cheating to just get a rough idea of what
      # would happen with a robust bayesian nls. The prior we want is
      # a prior distribution on alpha based on the entire plate above.
      # Incorporate prior by adding 1 or 2 data points to capture prior
      # belief for alpha. Assume that at 10 dilution steps we reach
      # alpha = min signal
      prior.rep <- 20
      dum <- data.frame(Y=c(yval[items], rep(p.alpha, prior.rep)),
                        Step=c(steps[items], rep(-10, prior.rep)))
      dum$p.beta <- p.beta
      dum$p.gamma <- p.gamma
      tmp <- try(nls(Y ~ s.alpha + p.beta*.ea(p.gamma*(Step+X)), data=dum,
                     start=list(X=passer[this], s.alpha=p.alpha),trace=trace))
    } else if (method=='pure') { # not bayesian
      dum <- data.frame(Y=yval[items], Steps=steps[items])
      dum$alpha <- p.alpha
      dum$beta <- p.beta
      dum$gamma <- p.gamma
      tmp <- try(nls(Y ~ alpha + beta*.ea(gamma*(Steps+X)), data=dum,
                     start=list(X=passer[this]),trace=trace))

    } else { # not bayesian and not pure
      # Now we can do a linear regression on
      # ln[z/(1-z)] / gamma = Step + X,  where z = (y-alpha)/beta which
      # should be equivalent to the non-linear
      #     Y ~ p.alpha + p.beta*ea(p.gamma*(Step+X))
      dum <- data.frame(Y=logitz[items],Step=steps[items])
      options(warn=warnLevel)
      tmp <- try(rlm(Y ~ 1 + offset(Step), data=dum, na.action='na.omit'))
      options(warn=0)
    } # end if(bayesian)

    if (is(tmp, 'try-error')) {
      warning('unavoidable nls/rlm error')
      warn2[this] <- paste(warn2[this], 'unavoidable nls-error')
      pass2[this] <- passer[this]
      pval2[this] <- 0.5 # magic number
      ss.ratio[this] <- 0      # magic number
    } else { # the model fitting succeded, so we can continue
      # start by recording the EC50 estimate for this series
      if (bayesian) {
        if(veryVerbose) {
          cat(paste('bayesian; X =', coef(tmp)['X']),
              '   coef =', coef(tmp), '\n')
        }
        pass2[this] <- coef(tmp)['X']
      } else {
        if(veryVerbose) {
          cat(paste(this, 'not bayesian; coef =', coef(tmp), '\n'))
        }
        pass2[this] <- coef(tmp)
      }

      # tstat = coef / std. error
      # rlm does not give p-value in pval2[i] <- coef(summary(tmp))[4]
      # 
      tstat <- try(coef(summary(tmp))[1] / coef(summary(tmp))[2])
      if (is(tstat, 'try-error')) {
        pval2[this] = 0.2 # yet another magic number
      } else {
        pval2[this] <- pt(tstat, sum(items)-2)
      }
      # Compute R^2 as sum(r[i]^2) / sum((y[i]-mean(y))^2) = fraction of
      # variance explained for this series
      # ss.ratio = 1 - (sum(tmp$residuals*tmp$residuals) /
      #                 (var(dum$Y, na.rm=T)*(length(tmp$residuals)-1)))
      # Compute R^2 fraction of variance explained, relative to average
      # variation for all dilution series. Idea is that top and bottom
      # dilution series may have flat expressions. We do not want to
      # penalize these as being "bad" fits. We just want to see if the
      # noise for that dilution series is "relatively" low. A typical
      # cutoff that we use for this value is that if the fraction of
      # the variance explained is less than 10%, then the dilution
      # series may be bad.
      if (ssy < 0) {
        ssy <- mean(unlist(lapply(series, function(x, ld, se) {
          items <- se==x
          var(ld[items], na.rm=TRUE)
        }, logitz, names(design))), na.rm=TRUE)
      } 
      ss.ratio[this] = (sum(tmp$residuals*tmp$residuals) /
                        (ssy*(length(tmp$residuals)-1)))
    } # end if(is(tmp, 'try-error'))
  } # end for (this in series)

  if(verbose) {
    cat('Finished estimating EC50 values.  Coefficients:\n')
    print(summary(pass2))
    cat('SS Ratio:\n')
    print(summary(ss.ratio))
  }

#  p.alpha <- unlist(p.alpha)
#  p.beta <- unlist(p.beta)
  names(p.gamma) <- NULL
  result <- new("RPPAFit",
                call=call, rppa=rppa, design=design, # inputs
                measure=measure, method=method,      # inputs
                coefficients=c(alpha=p.alpha, beta=p.beta, gamma=p.gamma),
                concentrations=pass2, lower=pass2, upper=pass2,
                intensities=p.alpha + p.beta*.ea(p.gamma*(pass2)),
                linear.p=myScreen$pvalue, slope=myScreen$slope,
                ss.ratio=ss.ratio, p.values=pval2, conf.width=0,
                warn=warn2,
                version=packageDescription("SuperCurve", fields="Version"))

  if(ci) { # compute confidence intervals for the estimates
    if(verbose) cat("Computing confidence intervals...\n")
    result <- getConfidenceInterval(result, design)
  } # end if(ci)

  
  result
}

.dilScreen <- function(data, design) {
  # Screen out dilution series that do not have at least a rough linear
  # fit. This will chop any series that have large anomolous values.

  # 'data' is a vector of intensity values selected from the MicroVigene values
  # 'design' is a RPPADesign object
  # the length of data should equal the number of rows in design@layout, and
  # the order should be the same.
  series <- seriesNames(design)   # omits the control spots automagically
  slope <- rep(0, length(series)) # initialize the return values
  names(slope) <- series          # use named vectors to ensure that things match 
  intercept <- slope
  pvalue <- 1 - slope
  for (this in series) {
    items <- design@layout$Series == this
    dum <- data.frame(B=design@layout$Steps[items], X=data[items])
    tmp <- lm(X ~ B, data=dum)
    intercept[this] <- coef(tmp)[1]
    slope[this]     <- coef(tmp)[2]
    pvalue[this]    <- coef(summary(tmp))[2,4]
  }
  list(slope=slope, intercept=intercept, pvalue=pvalue)
}

# compute a multiple of the logit transform.
# in practice, epsilon never changes.
.calcLogitz <- function(data, alpha, beta, gamma, epsilon=0.001) {
  z <- (data - alpha) / beta
  # Truncate points at min and max for dilution curve
  z[z < epsilon] <- epsilon
  z[z > (1-epsilon)] <- 1-epsilon
  log(z / (1-z)) / gamma
}

.firstPass <- function(yval, design, ignoreNegative, epsilon=1e-4) {
  # Computes crude estimates of the parameters (i.e., alpha, beta,
  # gamma, and the EC50s) so we can initialize the routine.

  # 'yval' is a vector of intensit values selected from the MircoVigene file
  # 'design' is a RPPADesign object. The length of 'yval' should match the
  # number of rows in design@layout, and be in the same order
  # in practice, 'epsilon' never changes

  if (ignoreNegative) {
    yval[yval<0] <- NA
  }
  temp <- yval[!.controlVector(design)]
  # compute a robust estimate for alpha and beta
  # Tried to use robust 'rnls' from 'sfsmisc' to fit alpha and beta below
  # but found it was still too sensitive to outliers.
  # This is a crude way to get alpha and beta but it seems to be robust.
  lBot <- quantile(temp, probs=c(.05), na.rm=TRUE)
  lTop <- quantile(temp, probs=c(.95), na.rm=TRUE)

  lindata <- .calcLogitz(yval, lBot, lTop-lBot, 1)

  series <- seriesNames(design) # omits the control spots automagically

  # initial estimate of the logistic slope across the dilution steps
  # there should be a more comprehensible way to write this:
  gamma <- median(unlist(lapply(series, function(ser, dsn, ld) {
    items <- dsn@layout$Series == ser   # get the items in this diluton series
    steps <- dsn@layout$Steps[items]    # get the log2 steps for this series
    x <- ld[items]                      # get the transformed intensity values
    y <- x[!is.na(x) & !is.infinite(x)] # only use valid values
    (max(y) - min(y))/(max(steps) - min(steps)) # ychange / xchange
  }, design, lindata)))

  # for each sample, estimate the offset
  passer <- rep(NA, length(series))
  names(passer) <- series
  for (this in series) {
    items <- design@layout$Series == this
    passer[this] <- median(lindata[items]/gamma-design@layout$Steps[items], na.rm=TRUE) 
  }
  
  list(lBot=lBot, lTop=lTop, gamma=gamma, passer=passer)
}

.ea <- function(x) { exp(x)/(1+exp(x)) }  # note: rnls does not work with local functions
.lp <- function(p) log(p/(1-p))

getConfidenceInterval <- function(result, alpha=0.10, nSim=50) {
  cf <- as.list(coef(result))
  A <- cf$alpha
  B <- cf$beta
  G <- cf$gamma
  series <- seriesNames(result@design)
  steps <- getSteps(result@design)
  res <- resid(result)         # actual residuals on the intensity scale
  yval <- fitted(result, "Y")  # best fit of the intensities
  xval <- fitted(result, "X")  # best fit concentrations on the log2 scale
  # we assume the residuals vary smoothly with the concentration or intensity
  # so, we use loess to fit the absolute residuals as a function of the
  # fitted concentration
  lo <- loess(ares ~ xval, data.frame(ares=abs(res), xval=xval))
  # We assume the residuals locally satisfy R ~ N(0, sigma). Then the
  # expected value of |R| is sigma*sqrt(2/pi), so:
  sigma <- sqrt(pi/2)*fitted(lo)
  for (this in series) {
    items <- result@design@layout$Series == this
    xhat <- xval[items]
    yhat <- yval[items]
		
    sim <- rep(NA, nSim)
    for (j in 1:nSim) { 
      resid.boot <- rnorm(sum(items), 0, sigma[items]) # sample the residuals
      #add resid.boot to y_hat;  refit; 
      ysim <- yhat + resid.boot
      logzsim <- .calcLogitz(ysim, A, B, G)
      dum2 <- data.frame(Y=logzsim, Step=steps[items])
      tmp2 <- try(rlm(Y ~ 1 + offset(Step), data=dum2, na.action='na.omit'))
      if (!is(tmp2, 'try-error')) {
        sim[j] <- coef(tmp2)
      }
    }
    result@lower[this] <- quantile(sim, alpha/2, na.rm=TRUE)
    result@upper[this] <- quantile(sim, 1 - alpha/2, na.rm=TRUE)
  }
  result@conf.width <- 1-alpha
  result
}




# Calculate power to distinguish between high expression group and low expression group
#
powercalc <- function(n, dilnFit, upper=0.95, lower=0.50, nrep=1000, alpha=0.05) {
	
  # n = size of sample to be drawn
  # dilnFit is an object from the dilutionFit function
  # upper is the quantile of the EC50 on the upper part of the curve
  # lower is the quantile of the EC50 on the lower part of the curve
  # nrep is the number of simulations (samples) to do in order to compute the power
	 
	 
  # This function takes an X value and paramater values and computes the expected
  # intesity      
  logistic <- function(alpha, beta, gamma, x) {
    alpha+beta*.ea(gamma*x)
  }           
	
  # We take two EC50 --- one that is the lower-th (e.g., 50th) percentile 
  # and the other that is the upper-th (e.g., 95th) percentile. 
  med <- quantile(dilnFit@concentrations, lower)
  top <- quantile(dilnFit@concentrations, upper)
	
  # We find which sample has the value closest to the percentile that was computed
  # in the previous step.  Metopvaldval is the sample closest to the lower percentile (called
  # such because usually this value will be the median).  topval is the sample closest 
  # to the upper percentile.
  temp <- abs(dilnFit@concentrations - med)
  medval <- which(temp==min(temp))
	
  temp <- abs(dilnFit@concentrations - top)
  topval <- which(temp==min(temp))

  cf <- as.list(coef(dilnFit))
  A <- cf$alpha
  B <- cf$beta
  G <- cf$gamma
  series <- seriesNames(dilnFit@design)
  steps <- getSteps(dilnFit@design)
  res  <- resid(dilnFit)        # actual residuals on the intensity scale
  yval <- fitted(dilnFit, "Y")  # best fit of the intensities
  xval <- fitted(dilnFit, "X")  # best fit concentrations on the log2 scale
  # we assume the residuals vary smoothly with the concentration or intensity
  # so, we use loess to fit the absolute residuals as a function of the
  # fitted concentration
  lo <- loess(ares ~ xval, data.frame(ares=abs(res), xval=xval))
  # We assume the residuals locally satisfy R ~ N(0, sigma). Then the
  # expected value of |R| is sigma*sqrt(2/pi), so:
  sigma <- sqrt(pi/2)*fitted(lo)
  nSim <- 64
  allsamp <- matrix(NA, 2, nSim)
  for (i in 1:2) {
    this <- series[c(medval, topval)][i]
    items <- dilnFit@design@layout$Series == this
    xhat <- xval[items]
    yhat <- yval[items]
    for (j in 1:nSim) { 
      resid.boot <- rnorm(sum(items), 0, sigma[items]) # sample the residuals
      #add resid.boot to y_hat;  refit; 
      ysim <- yhat + resid.boot
      logzsim <- .calcLogitz(ysim, A, B, G)
      dum2 <- data.frame(Y=logzsim, Step=steps[items])
      tmp2 <- try(rlm(Y ~ 1 + offset(Step), data=dum2, na.action='na.omit'))
      allsamp[i,j] <- coef(tmp2)
    }
  }

  # From the simulations done to estimate the confidence intervals of the EC50 values
  # we estimate the distribution of X values at percentile values found above.
  meddist <- density(allsamp[1,], n=1024)
  topdist <- density(allsamp[2,], n=1024)
	
  # From the distribution for X values, we find the distribution of Y values--that is
  # intensities--we expect to see at the percentile values.  The intensity values are
  # found using the supercurve and the paramater estimated in dilutionFit.
  med.int <- logistic(A, B, G, meddist$x)
  top.int <- logistic(A, B, G, topdist$x)
	
  # Now we draw a sample of the intensities from each of the distributions.
  # The probabilities associated with the intensities come from the probability
  # of drawing an X value. We use the wilcoxon test to test the difference
  # between the groups and store the p-value. This is repeated nrep (1000) times.
  mypval <- rep(NA,nrep)
  for (i in 1:nrep) {
    medsamp <- sample(med.int, n, prob=meddist$y)
    topsamp <- sample(top.int, n, prob=topdist$y)
    mypval[i] <- wilcox.test(medsamp,topsamp)$p.value
  }
	
  # The power is the number of p-values less than 0.05 (reject Ho) divided by nrep
  mypower <- sum(mypval<alpha)/nrep
  return(mypower)
}

